{% load i18n static %}
<link rel="stylesheet" href="{% static 'maykin_common/css/version.css' %}">
<div class="version" title="Git SHA: {{ GIT_SHA|default:'' }}">
    <span class="version__release">
        {% blocktrans %}version {{ RELEASE }}{% endblocktrans %}
    </span>
    {% if GIT_SHA %}
        <code class="version__git-hash">
            {% blocktrans %}GIT SHA: {{ GIT_SHA }}{% endblocktrans %}
        </code>
    {% endif %}
</div>
